'use strict';

const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');
const FriendlyErrorsPlugin = require('friendly-errors-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
// const OptimizeCSSPlugin = require('optimize-css-assets-webpack-plugin');
const HappyPack = require('happypack');

const rootDir = __dirname;
const wechatDir = path.join(rootDir, './demo-wxapp');
const distPath = path.resolve(__dirname, './wechatdist');
const subPackagePath = path.resolve(distPath, './wechatdist');

const jsPreloader = require.resolve('billund-next-core/lib/build/loader/vue-preloader/script');
const htmlPreloader = require.resolve('billund-next-core/lib/build/loader/vue-preloader/template');
const px2rpxLoader = {
    loader: 'px2rpx-loader',
    options: {
        baseDpr: 1,
        rpxUnit: 0.5
    }
};

const appEntry = {
    app: path.resolve(wechatDir, './app.wxapp.json')
};
const pageEntry = {
    'pages/index/index': path.resolve(wechatDir, './pages/index/index.wxpage.json'),
    'pages/second/second': path.resolve(wechatDir, './pages/second/second.wxpage.json')
};
const entries = Object.assign({}, appEntry, pageEntry);

module.exports = {
    entry: entries,
    target: require('mpvue-webpack-target'),
    output: {
        path: distPath,
        filename: 'static/js/[name].js',
        chunkFilename: 'static/js/[id].js',
        publicPath: distPath
    },
    resolve: {
        extensions: ['.js', '.vue', '.json'],
        alias: {
            vue: 'mpvue',
            '@': rootDir
        },
        symlinks: false,
        mainFields: ['mpvue', 'wechat', 'browser', 'module', 'main']
    },
    module: {
        rules: [{
            test: new RegExp('\.wxapp\.json$'),
            use: [{
                loader: require.resolve('babel-loader'),
                options: {
                    babelrc: false,
                    presets: [
                        ['env', {}], 'stage-0'
                    ],
                    plugins: [
                        'add-module-exports',
                        'transform-object-assign',
                        'array-includes',
                        'transform-runtime'
                    ]
                }
            }, {
                loader: 'mpvue-loader',
                options: {
                    checkMPEntry: true
                }
            }, {
                loader: require.resolve('billund-next-core/lib/build/loader/wxapp-config-loader/app')
            }]
        }, {
            test: new RegExp('\.wxpage\.json$'),
            use: [{
                loader: require.resolve('babel-loader'),
                options: {
                    babelrc: false,
                    presets: [
                        ['env', {}], 'stage-0'
                    ],
                    plugins: [
                        'add-module-exports',
                        'transform-object-assign',
                        'array-includes',
                        'transform-runtime'
                    ]
                }
            }, {
                loader: 'mpvue-loader',
                options: {
                    checkMPEntry: true
                }
            }, {
                loader: require.resolve('billund-next-core/lib/build/loader/wxapp-config-loader/page')
            }]
        }, {
            test: new RegExp('\.pageconfig\.json$'),
            use: [{
                loader: 'mpvue-loader',
                options: {
                    loaders: {
                        js: {
                            loader: require.resolve('babel-loader'),
                            options: {
                                babelrc: false,
                                presets: [
                                    ['env', {}], 'stage-0'
                                ],
                                plugins: [
                                    'add-module-exports',
                                    'transform-object-assign',
                                    'array-includes',
                                    'transform-runtime'
                                ]
                            }
                        },
                        less: ExtractTextPlugin.extract({
                            use: [{
                                    loader: 'css-loader',
                                    options: {
                                        importLoaders: 1
                                    }
                                },
                                'less-loader',
                                px2rpxLoader
                            ],
                            fallback: 'style-loader'
                        }),
                        css: ExtractTextPlugin.extract({
                            use: [{
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 1
                                }
                            }, px2rpxLoader],
                            fallback: 'style-loader'
                        })
                    },
                    preLoaders: {
                        js: jsPreloader,
                        html: htmlPreloader
                    }
                }
            }, {
                loader: require.resolve('billund-next-core/lib/build/loader/page-config-loader/index')
            }]
        }, {
            test: new RegExp('\.widget\.json$'),
            use: [{
                loader: 'mpvue-loader',
                options: {
                    loaders: {
                        js: {
                            loader: require.resolve('babel-loader'),
                            options: {
                                babelrc: false,
                                presets: [
                                    ['env', {}], 'stage-0'
                                ],
                                plugins: [
                                    'add-module-exports',
                                    'transform-object-assign',
                                    'array-includes',
                                    'transform-runtime'
                                ]
                            }
                        },
                        less: ExtractTextPlugin.extract({
                            use: [{
                                    loader: 'css-loader',
                                    options: {
                                        importLoaders: 1
                                    }
                                },
                                'less-loader',
                                px2rpxLoader
                            ],
                            fallback: 'style-loader'
                        }),
                        css: ExtractTextPlugin.extract({
                            use: [{
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 1
                                }
                            }, px2rpxLoader],
                            fallback: 'style-loader'
                        })
                    },
                    preLoaders: {
                        js: jsPreloader,
                        html: htmlPreloader
                    }
                }
            }, {
                loader: require.resolve('billund-next-core/lib/build/loader/widget-loader/index')
            }]
        }, {
            test: /\.vue$/,
            use: [{
                loader: 'mpvue-loader',
                options: {
                    loaders: {
                        js: {
                            loader: require.resolve('babel-loader'),
                            options: {
                                babelrc: false,
                                presets: [
                                    ['env', {}], 'stage-0'
                                ],
                                plugins: [
                                    'add-module-exports',
                                    'transform-object-assign',
                                    'array-includes',
                                    'transform-runtime'
                                ]
                            }
                        },
                        less: ExtractTextPlugin.extract({
                            use: [{
                                    loader: 'css-loader',
                                    options: {
                                        importLoaders: 1
                                    }
                                },
                                'less-loader',
                                px2rpxLoader
                            ],
                            fallback: 'style-loader'
                        }),
                        css: ExtractTextPlugin.extract({
                            use: [{
                                loader: 'css-loader',
                                options: {
                                    importLoaders: 1
                                }
                            }, px2rpxLoader],
                            fallback: 'style-loader'
                        })
                    },
                    preLoaders: {
                        js: jsPreloader,
                        html: htmlPreloader
                    }
                }
            }, {
                loader: require.resolve('billund-next-core/lib/build/loader/enhanced-vue-preloader/index'),
                options: {
                    widgetRegExp: '\.widget\.json$'
                }
            }]
        }, {
            test: /\.(js)$/,
            exclude: /node_modules/,
            use: {
                loader: require.resolve('babel-loader'),
                options: {
                    babelrc: false,
                    presets: [
                        ['env', {}], 'stage-0'
                    ],
                    plugins: [
                        'add-module-exports',
                        'transform-object-assign',
                        'array-includes',
                        'transform-runtime'
                    ]
                }
            }
        }, {
            test: /\.less$/,
            use: ExtractTextPlugin.extract({
                use: [{
                        loader: 'css-loader',
                        options: {
                            importLoaders: 1
                        }
                    },
                    'less-loader',
                    px2rpxLoader
                ],
                fallback: 'style-loader'
            })
        }, {
            test: /\.css$/,
            use: ExtractTextPlugin.extract({
                use: [{
                    loader: 'css-loader',
                    options: {
                        importLoaders: 1
                    }
                }, px2rpxLoader],
                fallback: 'style-loader'
            })
        }, {
            test: /\.(png|jpe?g|gif|svg)(\?.*)?$/,
            loader: 'url-loader',
            options: {
                limit: 10000,
                name: 'static/img/[name].[ext]' // TODO 这是微信要求的么？
            }
        }, {
            test: /\.(mp4|webm|ogg|mp3|wav|flac|aac)(\?.*)?$/,
            loader: 'url-loader',
            options: {
                limit: 10000,
                name: 'static/media/[name].[ext]' // TODO 这是微信要求的么？path.posix.join(subPackagePath, 'media/[name]].[ext]')
            }
        }, {
            test: /\.(woff2?|eot|ttf|otf)(\?.*)?$/,
            loader: 'url-loader',
            options: {
                limit: 10000,
                name: 'static/fonts/[name].[ext]' // TODO 这是微信要求的么？path.posix.join(subPackagePath, 'fonts/[name].[ext]')
            }
        }]
    },
    plugins: [
        new webpack.DefinePlugin({
            'process.env.NODE_ENV': JSON.stringify('dev'),
            'process.env.VUE_ENV': '"server"',
            'process.env': JSON.stringify('dev')
        }),
        // new HappyPack({
        //     cache: true,
        //     loaders: [{
        //         loader: require.resolve('babel-loader'),
        //         options: {
        //             babelrc: false,
        //             presets: [
        //                 ['env', {}], 'stage-0'
        //             ],
        //             plugins: [
        //                 'add-module-exports',
        //                 'transform-object-assign',
        //                 'array-includes',
        //                 'transform-runtime'
        //             ]
        //         }
        //     }]
        // }),
        // copy from ./webpack.prod.conf.js
        // extract css into its own file
        new ExtractTextPlugin({
            // filename: utils.assetsPath('css/[name].[contenthash].css')
            filename: 'static/css/[name].wxss'
        }),
        // Compress extracted CSS. We are using this plugin so that possible
        // duplicated CSS from different components can be deduped.
        // new OptimizeCSSPlugin({
        //     cssProcessorOptions: {
        //         safe: true
        //     }
        // }),
        new webpack.optimize.CommonsChunkPlugin({
            name: 'vendor',
            minChunks(module, count) {
                // any required modules inside node_modules are extracted to vendor
                return (
                    module.resource &&
                    /\.js$/.test(module.resource) &&
                    module.resource.indexOf('node_modules') >= 0
                )
            }
        }),
        new webpack.optimize.CommonsChunkPlugin({
            name: 'manifest',
            chunks: ['vendor']
        }),
        // copy custom static assets
        new CopyWebpackPlugin([{
            from: path.resolve(rootDir, './static'),
            to: subPackagePath,
            ignore: ['.*']
        }]),

        // https://github.com/glenjamin/webpack-hot-middleware#installation--usage
        // new webpack.HotModuleReplacementPlugin(),
        new webpack.NoEmitOnErrorsPlugin(),
        // https://github.com/ampedandwired/html-webpack-plugin
        // new HtmlWebpackPlugin({
        //   filename: 'index.html',
        //   template: 'index.html',
        //   inject: true
        // }),
        new FriendlyErrorsPlugin()
    ]
};